<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0" 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:typ="http://unigrids.org/2006/04/types">

  <xsl:decimal-format grouping-separator=" "/>
  <xsl:variable name="numPattern">### ###</xsl:variable>
  <xsl:output method="html" encoding="utf-8"/>
  <xsl:template match="/">
    <xsl:variable name="isDir" select="contains(
        translate(.//typ:IsDirectory,'TRUE','true'),'true')"/>
    <html>
      <head>
        <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
      </head>
      <body>
        <xsl:if test="$isDir">
          <h2>Directory: <xsl:call-template name="GetFileName">
                         <xsl:with-param name="pathPart" select="translate(.//typ:Path,'\','/')"/>
                       </xsl:call-template>
          </h2>
        </xsl:if>
        <xsl:if test="not($isDir)">
          <h2>File: <xsl:call-template name="GetFileName">
                      <xsl:with-param name="pathPart" select="translate(.//typ:Path,'\','/')"/>
                    </xsl:call-template>
          </h2>
        </xsl:if>
        <table border="1">
          <tr bgcolor="#D0E5F8">
            <th align="left">Property</th>
            <th align="left">Value</th>
          </tr>
          <tr>
            <td nowrap="nowrap">Path in the storage</td>
            <td nowrap="nowrap"><xsl:value-of select=".//typ:Path"/></td>
          </tr>
          <tr>
            <td nowrap="nowrap">Size [bytes]</td>
            <td nowrap="nowrap">
              <xsl:call-template name="NumberFormat">
                <xsl:with-param name="num" select=".//typ:Size"/>
              </xsl:call-template>
            </td>
          </tr>
          <tr>
            <td nowrap="nowrap">Modification time</td>
            <td nowrap="nowrap">
              <xsl:call-template name="DateTimeFormatter">
                <xsl:with-param name="datetime" select=".//typ:LastModified"/>
              </xsl:call-template>
            </td>
          </tr>
          <xsl:for-each select=".//typ:Permissions">
            <tr>
              <td nowrap="nowrap">Executable</td>
              <td nowrap="nowrap">
                <xsl:value-of select="typ:Executable"/>
              </td>
            </tr>
            <tr>
              <td nowrap="nowrap">Writable</td>
              <td nowrap="nowrap">
                <xsl:value-of select="typ:Writable"/>
              </td>
            </tr>
            <tr>
              <td nowrap="nowrap">Readable</td>
              <td nowrap="nowrap">
                <xsl:value-of select="typ:Readable"/>
              </td>
            </tr>
          </xsl:for-each>
        </table>
      </body>
    </html>
  </xsl:template>

  <xsl:template name="GetFileName">
    <!-- Извлечение имени файла из строки пути -->
    <xsl:param name="pathPart"/>
    <xsl:variable name="isSeparator" select="contains($pathPart,'/')"/>
    <!-- Если ещё есть разделители - ракурсивно идём дальше -->
    <xsl:if test="$isSeparator">
      <xsl:variable name="nextPart" select="substring-after($pathPart,'/')"/>
      <xsl:call-template name="GetFileName">
        <xsl:with-param name="pathPart" select="$nextPart"/>
      </xsl:call-template>
    </xsl:if>
    <!-- Если разделителей больше нет - вывод результата -->
    <xsl:if test="not($isSeparator)">
      <xsl:value-of select="$pathPart"/>
    </xsl:if>
  </xsl:template>
  
  <xsl:template name="NumberFormat">
    <!-- Округление чисел и разделение групп разрядов -->
    <xsl:param name="num"/>
    <xsl:if test="string-length(string($num))>0">
      <xsl:value-of select="format-number(round($num),$numPattern)"/>
    </xsl:if>
  </xsl:template>

  <xsl:template name="DateTimeFormatter">
    <!-- Преобразование даты из строки dateTime "1999-01-31T17:30:06.111+03:00" -->
    <!-- к читабельному виду "1999-01-31 17:30:06 UTC+03:00" -->
    <!-- Разбор строки даты-времени типа "dateTime" в соответствии с -->
    <!-- http://www.w3.org/TR/xmlschema-2/ -->
    <xsl:param name="datetime">' '</xsl:param>
    <!-- работа только если на входе непустая строка -->
    <xsl:if test="string-length($datetime)>0">
      <xsl:variable name="bigDatetime" select="translate($datetime,'tz','TZ')"/>
      <xsl:variable name="date" select="substring-before($bigDatetime,'T')"/>
      <xsl:variable name="time" select="substring-after($bigDatetime,'T')"/>
      <xsl:variable name="isTimezone" select="contains($time,'+') or contains($time,'-') or contains($time,'Z')"/>
    <!-- печать отформатированной даты -->
      <xsl:call-template name="LocalDateFormat">
        <xsl:with-param name="date" select="$date"/>
      </xsl:call-template>
    <!-- пробел и затем отформатированное время -->  
      <xsl:text> </xsl:text>
    <!-- если часовой пояс не указан - печать времени без изменений -->
      <xsl:if test="not($isTimezone)">
        <xsl:call-template name="TimeWithoutMs">
          <xsl:with-param name="time" select="$time"/>
        </xsl:call-template>
      </xsl:if>
    <!-- если часовой пояс есть - приведение хвоста к виду "UTC..." -->
      <xsl:if test="$isTimezone">
        <xsl:if test="contains($time,'+')">
          <xsl:call-template name="TimeWithoutMs">
            <xsl:with-param name="time" select="substring-before($time,'+')"/>
          </xsl:call-template>
          <xsl:text> UTC+</xsl:text>
          <xsl:value-of select="substring-after($time,'+')"/>
        </xsl:if>
        <xsl:if test="contains($time,'-')">
          <xsl:call-template name="TimeWithoutMs">
            <xsl:with-param name="time" select="substring-before($time,'-')"/>
          </xsl:call-template>
          <xsl:text> UTC-</xsl:text>
          <xsl:value-of select="substring-after($time,'-')"/>
        </xsl:if>
      <!-- если не "+" или "-", а только "Z", это значит "UTC+00:00", и поэтому просто время и "UTC" -->
        <xsl:if test="contains($time,'Z')">
          <xsl:call-template name="TimeWithoutMs">
            <xsl:with-param name="time" select="substring-before($time,'Z')"/>
          </xsl:call-template>
          <xsl:text> UTC</xsl:text>
        </xsl:if>
      </xsl:if>
    </xsl:if>
  </xsl:template>

  <xsl:template name="TimeWithoutMs">
    <!-- Удаление милилсекунд в конце строки времени" -->
    <xsl:param name="time">' '</xsl:param>
    <xsl:variable name="isMilliseconds" select="contains($time,'.')"/>
    <!-- если миллисекунды есть - убрать их -->
    <xsl:if test="$isMilliseconds">
      <xsl:value-of select="substring-before($time,'.')"/>
    </xsl:if>
    <!-- если миллисекунд нет - возврат всей строки -->
    <xsl:if test="not($isMilliseconds)">
      <xsl:value-of select="$time"/>
    </xsl:if>
  </xsl:template>

  <xsl:template name="LocalDateFormat">
    <!-- форматирование даты к виду "ДД/ММ/ГГГ" -->
    <xsl:param name="date"/>
    <xsl:variable name='firstMinus' select="starts-with($date,'-')"/>
    <xsl:if test="$firstMinus">
      <!-- есть первый минус - до Р.Х. -->
      <xsl:text>до Р.Х. </xsl:text>
      <!-- формат даты -->
      <xsl:call-template name="LocalDateFormat2">
        <xsl:with-param name="date" select="substring-after($date,'-')"/>
      </xsl:call-template>
    </xsl:if>
    <xsl:if test="not($firstMinus)">
      <!-- формат даты -->
      <xsl:call-template name="LocalDateFormat2">
        <xsl:with-param name="date" select="$date"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <xsl:template name="LocalDateFormat2">
    <!-- форматирование даты к виду "ДД/ММ/ГГГ" -->
    <xsl:param name="date"/>
    <xsl:variable name='year' select="substring-before($date,'-')"/>
    <xsl:variable name='monthDay' select="substring-after($date,'-')"/>
    <xsl:variable name='month' select="substring-before($monthDay,'-')"/>
    <xsl:variable name='day' select="substring-after($monthDay,'-')"/>
    <xsl:value-of select="concat($day,'/',$month,'/',$year)"/>
  </xsl:template>

</xsl:stylesheet>
