<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0" 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:rl="http://docs.oasis-open.org/wsrf/rl-2"
	xmlns:jsdl="http://schemas.ggf.org/jsdl/2005/11/jsdl"
	xmlns:typ="http://unigrids.org/2006/04/types"
	xmlns:sms="http://unigrids.org/2006/04/services/sms">

  <xsl:decimal-format grouping-separator=" "/>
  <xsl:variable name="numPattern">### ###</xsl:variable>
  <xsl:output method="html" encoding="utf-8"/>
  <xsl:template match="sms:StorageProperties">
    <html>
      <head>
        <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
      </head>
      <body>
        <h2>Хранилище: <xsl:value-of select="jsdl:FileSystem/@name"/></h2>
        <!-- Таблица основных свойств -->
        <table border="1">
          <tr bgcolor="#D0E5F8">
            <th align="left">Свойство</th>
            <th align="left">Значение</th>
          </tr>
          <tr>
            <td nowrap="nowrap">Тип файловой системы</td>
            <td nowrap="nowrap"><xsl:value-of select="jsdl:FileSystem/jsdl:FileSystemType"/></td>
          </tr>
          <!-- если на входе непустая строка -->
		    <xsl:variable name="descr" select="jsdl:FileSystem/jsdl:Description"/>
          <xsl:if test="string-length($descr)>0">
            <tr>
              <td nowrap="nowrap">Описание</td>
              <td nowrap="nowrap"><xsl:value-of select="$descr"/></td>
            </tr>
          </xsl:if>
          <!-- если на входе непустая строка -->
		    <xsl:variable name="ver" select="typ:Version"/>
          <xsl:if test="string-length($ver)>0">
            <tr>
              <td nowrap="nowrap">Версия</td>
              <td nowrap="nowrap"><xsl:value-of select="$ver"/></td>
            </tr>
          </xsl:if>
          <!-- если на входе непустая строка -->
		    <xsl:variable name="mount" select="jsdl:FileSystem/jsdl:MountPoint"/>
          <xsl:if test="string-length($mount)>0">
            <tr>
              <td nowrap="nowrap">Точка монтирования</td>
              <td nowrap="nowrap"><xsl:value-of select="$mount"/></td>
            </tr>
          </xsl:if>
          <!-- если на входе непустая строка -->
		    <xsl:variable name="freeSpace" select="jsdl:FileSystem/jsdl:DiskSpace/jsdl:Exact"/>
          <xsl:if test="string-length($freeSpace)>0">
            <xsl:variable name="num" select="$freeSpace"/>
            <!-- печать отформатированной размера свободного места -->
            <tr>
              <td nowrap="nowrap">Свободное пространство [байт]</td>
              <td nowrap="nowrap">
                <xsl:call-template name="FloatNumberFormat">
                  <xsl:with-param name="float" select="$freeSpace"/>
                </xsl:call-template>
              </td>
            </tr>
          </xsl:if>
          <tr>
            <td nowrap="nowrap">Текущее время</td>
            <td nowrap="nowrap">
              <xsl:call-template name="DateTimeFormatter">
                <xsl:with-param name="datetime" select="rl:CurrentTime"/>
              </xsl:call-template>
            </td>
          </tr>
           <tr>
            <td nowrap="nowrap">Время окончания жизни</td>
            <td nowrap="nowrap">
              <xsl:call-template name="DateTimeFormatter">
                <xsl:with-param name="datetime" select="rl:TerminationTime"/>
              </xsl:call-template>
            </td>
          </tr>
        </table>
        <!-- зазор между таблицами -->
        <br/>
        <!-- Таблица файловых протоколов -->
        <table border="1">
          <tr bgcolor="#D0E5F8">
            <th align="left">Поддерживаемые файловые протоколы</th>
          </tr>
          <xsl:for-each select="typ:Protocol">
            <tr>
              <td nowrap="nowrap"><xsl:value-of select="."/></td>
            </tr>
          </xsl:for-each>
        </table>
      </body>
    </html>
  </xsl:template>

  <xsl:template name="FloatNumberFormat">
    <!-- форматирование числа формата с плавающей точкой -->
    <xsl:param name="float"/>
    <xsl:variable name='num' select="translate($float,'e','E')"/>
    <xsl:variable name='basenum' select="number(substring-before($float,'E'))"/>
    <xsl:variable name='exp' select="substring-after($float,'E')"/>
    <xsl:choose>
      <xsl:when test ="string-length($exp)>0">
        <!-- есть экспонента в числе -->
        <xsl:variable name='expnum' select="round(number($exp))"/>
        <xsl:choose>
          <xsl:when test ="$expnum=0">
            <xsl:call-template name="NumberFormat">
              <xsl:with-param name="num" select="$basenum"/>
            </xsl:call-template>
          </xsl:when> 
          <xsl:when test ="$expnum=1">
            <xsl:call-template name="NumberFormat">
              <xsl:with-param name="num" select="$basenum*10"/>
            </xsl:call-template>
          </xsl:when> 
          <xsl:when test ="$expnum=2">
            <xsl:call-template name="NumberFormat">
              <xsl:with-param name="num" select="$basenum*100"/>
            </xsl:call-template>
          </xsl:when> 
          <xsl:when test ="$expnum=3">
            <xsl:call-template name="NumberFormat">
              <xsl:with-param name="num" select="$basenum*1000"/>
            </xsl:call-template>
          </xsl:when> 
          <xsl:when test ="$expnum=4">
            <xsl:call-template name="NumberFormat">
              <xsl:with-param name="num" select="$basenum*10000"/>
            </xsl:call-template>
          </xsl:when> 
          <xsl:when test ="$expnum=5">
            <xsl:call-template name="NumberFormat">
              <xsl:with-param name="num" select="$basenum*100000"/>
            </xsl:call-template>
          </xsl:when> 
          <xsl:when test ="$expnum=6">
            <xsl:call-template name="NumberFormat">
              <xsl:with-param name="num" select="$basenum*1000000"/>
            </xsl:call-template>
          </xsl:when> 
          <xsl:when test ="$expnum=7">
            <xsl:call-template name="NumberFormat">
              <xsl:with-param name="num" select="$basenum*10000000"/>
            </xsl:call-template>
          </xsl:when> 
          <xsl:when test ="$expnum=8">
            <xsl:call-template name="NumberFormat">
              <xsl:with-param name="num" select="$basenum*100000000"/>
            </xsl:call-template>
          </xsl:when> 
          <xsl:when test ="$expnum=9">
            <xsl:call-template name="NumberFormat">
              <xsl:with-param name="num" select="$basenum*1000000000"/>
            </xsl:call-template>
          </xsl:when> 
          <xsl:when test ="$expnum=10">
            <xsl:call-template name="NumberFormat">
              <xsl:with-param name="num" select="$basenum*10000000000"/>
            </xsl:call-template>
          </xsl:when> 
          <xsl:when test ="$expnum=11">
            <xsl:call-template name="NumberFormat">
              <xsl:with-param name="num" select="$basenum*100000000000"/>
            </xsl:call-template>
          </xsl:when> 
          <xsl:when test ="$expnum=12">
            <xsl:call-template name="NumberFormat">
              <xsl:with-param name="num" select="$basenum*1000000000000"/>
            </xsl:call-template>
          </xsl:when> 
          <xsl:when test ="$expnum=13">
            <xsl:call-template name="NumberFormat">
              <xsl:with-param name="num" select="$basenum*10000000000000"/>
            </xsl:call-template>
          </xsl:when> 
          <xsl:when test ="$expnum=14">
            <xsl:call-template name="NumberFormat">
              <xsl:with-param name="num" select="$basenum*100000000000000"/>
            </xsl:call-template>
          </xsl:when> 
          <xsl:when test ="$expnum=15">
            <xsl:call-template name="NumberFormat">
              <xsl:with-param name="num" select="$basenum*1000000000000000"/>
            </xsl:call-template>
          </xsl:when> 
          <xsl:otherwise>
            <!-- неизвестно что -->
            <xsl:value-of select="$float"/>
          </xsl:otherwise>
        </xsl:choose> 
      </xsl:when> 
      <xsl:otherwise>
        <!-- нет экспоненты в числе -->
        <xsl:call-template name="NumberFormat">
          <xsl:with-param name="num" select="$basenum"/>
        </xsl:call-template>
      </xsl:otherwise> 
    </xsl:choose> 
  </xsl:template>

  <xsl:template name="NumberFormat">
    <!-- Округление чисел и разделение групп разрядов -->
    <xsl:param name="num"/>
    <xsl:if test="string-length(string($num))>0">
      <xsl:value-of select="format-number(round($num),$numPattern)"/>
    </xsl:if>
  </xsl:template>

  <xsl:template name="DateTimeFormatter">
    <!-- Преобразование даты из строки dateTime "1999-01-31T17:30:06.111+03:00" -->
    <!-- к читабельному виду "1999-01-31 17:30:06 UTC+03:00" -->
    <!-- Разбор строки даты-времени типа "dateTime" в соответствии с -->
    <!-- http://www.w3.org/TR/xmlschema-2/ -->
    <xsl:param name="datetime">' '</xsl:param>
    <!-- работа только если на входе непустая строка -->
    <xsl:if test="string-length($datetime)>0">
      <xsl:variable name="bigDatetime" select="translate($datetime,'tz','TZ')"/>
      <xsl:variable name="date" select="substring-before($bigDatetime,'T')"/>
      <xsl:variable name="time" select="substring-after($bigDatetime,'T')"/>
      <xsl:variable name="isTimezone" select="contains($time,'+') or contains($time,'-') or contains($time,'Z')"/>
    <!-- печать отформатированной даты -->
      <xsl:call-template name="LocalDateFormat">
        <xsl:with-param name="date" select="$date"/>
      </xsl:call-template>
    <!-- пробел и затем отформатированное время -->  
      <xsl:text> </xsl:text>
    <!-- если часовой пояс не указан - печать времени без изменений -->
      <xsl:if test="not($isTimezone)">
        <xsl:call-template name="TimeWithoutMs">
          <xsl:with-param name="time" select="$time"/>
        </xsl:call-template>
      </xsl:if>
    <!-- если часовой пояс есть - приведение хвоста к виду "UTC..." -->
      <xsl:if test="$isTimezone">
        <xsl:if test="contains($time,'+')">
          <xsl:call-template name="TimeWithoutMs">
            <xsl:with-param name="time" select="substring-before($time,'+')"/>
          </xsl:call-template>
          <xsl:text> UTC+</xsl:text>
          <xsl:value-of select="substring-after($time,'+')"/>
        </xsl:if>
        <xsl:if test="contains($time,'-')">
          <xsl:call-template name="TimeWithoutMs">
            <xsl:with-param name="time" select="substring-before($time,'-')"/>
          </xsl:call-template>
          <xsl:text> UTC-</xsl:text>
          <xsl:value-of select="substring-after($time,'-')"/>
        </xsl:if>
      <!-- если не "+" или "-", а только "Z", это значит "UTC+00:00", и поэтому просто время и "UTC" -->
        <xsl:if test="contains($time,'Z')">
          <xsl:call-template name="TimeWithoutMs">
            <xsl:with-param name="time" select="substring-before($time,'Z')"/>
          </xsl:call-template>
          <xsl:text> UTC</xsl:text>
        </xsl:if>
      </xsl:if>
    </xsl:if>
  </xsl:template>

  <xsl:template name="TimeWithoutMs">
    <!-- Удаление милилсекунд в конце строки времени" -->
    <xsl:param name="time">' '</xsl:param>
    <xsl:variable name="isMilliseconds" select="contains($time,'.')"/>
    <!-- если миллисекунды есть - убрать их -->
    <xsl:if test="$isMilliseconds">
      <xsl:value-of select="substring-before($time,'.')"/>
    </xsl:if>
    <!-- если миллисекунд нет - возврат всей строки -->
    <xsl:if test="not($isMilliseconds)">
      <xsl:value-of select="$time"/>
    </xsl:if>
  </xsl:template>

  <xsl:template name="LocalDateFormat">
    <!-- форматирование даты к виду "ДД.ММ.ГГГ" -->
    <xsl:param name="date"/>
    <xsl:variable name='firstMinus' select="starts-with($date,'-')"/>
    <xsl:if test="$firstMinus">
      <!-- есть первый минус - до Р.Х. -->
      <xsl:text>до Р.Х. </xsl:text>
      <!-- формат даты -->
      <xsl:call-template name="LocalDateFormat2">
        <xsl:with-param name="date" select="substring-after($date,'-')"/>
      </xsl:call-template>
    </xsl:if>
    <xsl:if test="not($firstMinus)">
      <!-- формат даты -->
      <xsl:call-template name="LocalDateFormat2">
        <xsl:with-param name="date" select="$date"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <xsl:template name="LocalDateFormat2">
    <!-- форматирование даты к виду "ДД.ММ.ГГГ" -->
    <xsl:param name="date"/>
    <xsl:variable name='year' select="substring-before($date,'-')"/>
    <xsl:variable name='monthDay' select="substring-after($date,'-')"/>
    <xsl:variable name='month' select="substring-before($monthDay,'-')"/>
    <xsl:variable name='day' select="substring-after($monthDay,'-')"/>
    <xsl:value-of select="concat($day,'.',$month,'.',$year)"/>
  </xsl:template>

</xsl:stylesheet>
