<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0" 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:tss="http://unigrids.org/2006/04/services/tss"
	xmlns:rl="http://docs.oasis-open.org/wsrf/rl-2"
	xmlns:jsdl="http://schemas.ggf.org/jsdl/2005/11/jsdl"
	xmlns:typ="http://unigrids.org/2006/04/types"
	xmlns:add="http://www.w3.org/2005/08/addressing">

  <xsl:decimal-format grouping-separator=" "/>
  <xsl:variable name="numPattern">### ###</xsl:variable>
  <xsl:output method="html" encoding="utf-8"/>

  <xsl:template match="tss:TargetSystemProperties">
    <html>
      <head>
        <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />
      </head>
      <body>
        <h2>Целевая система: <xsl:value-of select="tss:Name"/></h2>
        <table border="1">
          <tr bgcolor="#D0E5F8">
            <th align="left">Свойство</th>
            <th align="left">Значение</th>
          </tr>
          <!-- если на входе непустая строка -->
		    <xsl:variable name="ver" select="typ:Version"/>
          <xsl:if test="string-length($ver)>0">
            <tr>
              <td nowrap="nowrap">Версия</td>
              <td nowrap="nowrap"><xsl:value-of select="$ver"/></td>
            </tr>
          </xsl:if>
          <tr>
            <td nowrap="nowrap">Тип процессора</td>
            <td nowrap="nowrap"><xsl:value-of select="typ:Processor/jsdl:CPUArchitecture/jsdl:CPUArchitectureName"/></td>
          </tr>
          <tr>
            <td nowrap="nowrap">Операционная система</td>
            <td nowrap="nowrap"><xsl:value-of select="jsdl:OperatingSystem/jsdl:OperatingSystemType/jsdl:OperatingSystemName"/></td>
          </tr>
          <tr>
            <td nowrap="nowrap">Количество задач</td>
            <td nowrap="nowrap"><xsl:value-of select="tss:TotalNumberOfJobs"/></td>
          </tr>
          <tr>
            <td nowrap="nowrap">Время запуска</td>
            <td nowrap="nowrap">
              <xsl:call-template name="DateTimeFormatter">
                <xsl:with-param name="datetime" select="typ:UpSince"/>
              </xsl:call-template>
            </td>
          </tr>
          <tr>
            <td nowrap="nowrap">Текущее время</td>
            <td nowrap="nowrap">
              <xsl:call-template name="DateTimeFormatter">
                <xsl:with-param name="datetime" select="rl:CurrentTime"/>
              </xsl:call-template>
            </td>
          </tr>
           <tr>
            <td nowrap="nowrap">Время окончания жизни</td>
            <td nowrap="nowrap">
              <xsl:call-template name="DateTimeFormatter">
                <xsl:with-param name="datetime" select="rl:TerminationTime"/>
              </xsl:call-template>
            </td>
          </tr>
        </table>
      
        <h2>Производительность</h2>  
        <table border="1">
          <tr bgcolor="#D0E5F8">
            <th align="left">Свойство</th>
            <th align="left">Значение</th>
            <th align="left">Минимум</th>
            <th align="left">Максимум</th>
          </tr>
          <tr>
            <td nowrap="nowrap">Физическая память [МБ]</td>
            <td nowrap="nowrap">
              <xsl:call-template name="NumberFormat">
                <xsl:with-param name="num" select="jsdl:IndividualPhysicalMemory/jsdl:Exact div 1048576"/>
              </xsl:call-template>
            </td>
            <td nowrap="nowrap">
              <xsl:call-template name="NumberFormat">
                <xsl:with-param name="num" select="jsdl:IndividualPhysicalMemory/jsdl:Range/jsdl:LowerBound div 1048576"/>
              </xsl:call-template>
            </td>
            <td nowrap="nowrap">
              <xsl:call-template name="NumberFormat">
                <xsl:with-param name="num" select="jsdl:IndividualPhysicalMemory/jsdl:Range/jsdl:UpperBound div 1048576"/>
              </xsl:call-template>
            </td>
          </tr>
          <tr>
            <td nowrap="nowrap">Количество ресурсов</td>
            <td nowrap="nowrap">
              <xsl:call-template name="NumberFormat">
                <xsl:with-param name="num" select="jsdl:TotalResourceCount/jsdl:Exact"/>
              </xsl:call-template>
            </td>
            <td nowrap="nowrap">
              <xsl:call-template name="NumberFormat">
                <xsl:with-param name="num" select="jsdl:TotalResourceCount/jsdl:Range/jsdl:LowerBound"/>
              </xsl:call-template>
            </td>
            <td nowrap="nowrap">
              <xsl:call-template name="NumberFormat">
                <xsl:with-param name="num" select="jsdl:TotalResourceCount/jsdl:Range/jsdl:UpperBound"/>
              </xsl:call-template>
            </td>
          </tr>
          <tr>
            <td nowrap="nowrap">Количество процессоров</td>
            <td>
              <xsl:call-template name="NumberFormat">
                <xsl:with-param name="num" select="jsdl:IndividualCPUCount/jsdl:Exact"/>
              </xsl:call-template>
            </td>
            <td nowrap="nowrap">
              <xsl:call-template name="NumberFormat">
                <xsl:with-param name="num" select="jsdl:IndividualCPUCount/jsdl:Range/jsdl:LowerBound"/>
              </xsl:call-template>
            </td>
            <td nowrap="nowrap">
              <xsl:call-template name="NumberFormat">
                <xsl:with-param name="num" select="jsdl:IndividualCPUCount/jsdl:Range/jsdl:UpperBound"/>
              </xsl:call-template>
            </td>
          </tr>
          <tr>
            <td nowrap="nowrap">Процессорное время [сек]</td>
            <td nowrap="nowrap">
              <xsl:call-template name="NumberFormat">
                <xsl:with-param name="num" select="jsdl:IndividualCPUTime/jsdl:Exact"/>
              </xsl:call-template>
            </td>
            <td nowrap="nowrap">
              <xsl:call-template name="NumberFormat">
                <xsl:with-param name="num" select="jsdl:IndividualCPUTime/jsdl:Range/jsdl:LowerBound"/>
              </xsl:call-template>
            </td>
            <td nowrap="nowrap">
              <xsl:call-template name="NumberFormat">
                <xsl:with-param name="num" select="jsdl:IndividualCPUTime/jsdl:Range/jsdl:UpperBound"/>
              </xsl:call-template>
            </td>
          </tr>
        </table>
        
        <h2>Приложения</h2>  
        <table border="1">
          <tr bgcolor="#D0E5F8">
            <th align="left">Приложение</th>
            <th align="left">Версия</th>
            <th align="left">Описание</th>
          </tr>
          <xsl:for-each select="tss:ApplicationResource">
            <xsl:sort select="jsdl:ApplicationName"/>
            <tr>
              <td nowrap="nowrap"><xsl:value-of select="jsdl:ApplicationName"/></td>
              <td nowrap="nowrap"><xsl:value-of select="jsdl:ApplicationVersion"/></td>
              <td><xsl:value-of select="jsdl:Description"/></td>
            </tr>
          </xsl:for-each>
        </table>
        
        <xsl:if test="typ:StorageReference">
        <h2>Хранилища</h2>  
        <table border="1">
          <tr bgcolor="#D0E5F8">
            <th align="left">Тип</th>
            <th align="left">Ссылка конечной точки</th>
          </tr>
          <xsl:for-each select="typ:StorageReference">
            <tr>
              <td nowrap="nowrap"><xsl:value-of select="typ:Type"/></td>
              <td nowrap="nowrap"><xsl:value-of select="typ:StorageEndpointReference/add:Address"/></td>
            </tr>
          </xsl:for-each>
        </table>
        </xsl:if>
        
      </body>
    </html>
  </xsl:template>

  <xsl:template name="NumberFormat">
    <!-- Округление чисел и разделение групп разрядов -->
    <xsl:param name="num"/>
    <xsl:if test="string-length(string($num))>0">
      <xsl:value-of select="format-number(round($num),$numPattern)"/>
    </xsl:if>
  </xsl:template>

  <xsl:template name="DateTimeFormatter">
    <!-- Преобразование даты из строки dateTime "1999-01-31T17:30:06.111+03:00" -->
    <!-- к читабельному виду "1999-01-31 17:30:06 UTC+03:00" -->
    <!-- Разбор строки даты-времени типа "dateTime" в соответствии с -->
    <!-- http://www.w3.org/TR/xmlschema-2/ -->
    <xsl:param name="datetime">' '</xsl:param>
    <!-- работа только если на входе непустая строка -->
    <xsl:if test="string-length($datetime)>0">
      <xsl:variable name="bigDatetime" select="translate($datetime,'tz','TZ')"/>
      <xsl:variable name="date" select="substring-before($bigDatetime,'T')"/>
      <xsl:variable name="time" select="substring-after($bigDatetime,'T')"/>
      <xsl:variable name="isTimezone" select="contains($time,'+') or contains($time,'-') or contains($time,'Z')"/>
    <!-- печать отформатированной даты -->
      <xsl:call-template name="LocalDateFormat">
        <xsl:with-param name="date" select="$date"/>
      </xsl:call-template>
    <!-- пробел и затем отформатированное время -->  
      <xsl:text> </xsl:text>
    <!-- если часовой пояс не указан - печать времени без изменений -->
      <xsl:if test="not($isTimezone)">
        <xsl:call-template name="TimeWithoutMs">
          <xsl:with-param name="time" select="$time"/>
        </xsl:call-template>
      </xsl:if>
    <!-- если часовой пояс есть - приведение хвоста к виду "UTC..." -->
      <xsl:if test="$isTimezone">
        <xsl:if test="contains($time,'+')">
          <xsl:call-template name="TimeWithoutMs">
            <xsl:with-param name="time" select="substring-before($time,'+')"/>
          </xsl:call-template>
          <xsl:text> UTC+</xsl:text>
          <xsl:value-of select="substring-after($time,'+')"/>
        </xsl:if>
        <xsl:if test="contains($time,'-')">
          <xsl:call-template name="TimeWithoutMs">
            <xsl:with-param name="time" select="substring-before($time,'-')"/>
          </xsl:call-template>
          <xsl:text> UTC-</xsl:text>
          <xsl:value-of select="substring-after($time,'-')"/>
        </xsl:if>
      <!-- если не "+" или "-", а только "Z", это значит "UTC+00:00", и поэтому просто время и "UTC" -->
        <xsl:if test="contains($time,'Z')">
          <xsl:call-template name="TimeWithoutMs">
            <xsl:with-param name="time" select="substring-before($time,'Z')"/>
          </xsl:call-template>
          <xsl:text> UTC</xsl:text>
        </xsl:if>
      </xsl:if>
    </xsl:if>
  </xsl:template>

  <xsl:template name="TimeWithoutMs">
    <!-- Удаление милилсекунд в конце строки времени" -->
    <xsl:param name="time">' '</xsl:param>
    <xsl:variable name="isMilliseconds" select="contains($time,'.')"/>
    <!-- если миллисекунды есть - убрать их -->
    <xsl:if test="$isMilliseconds">
      <xsl:value-of select="substring-before($time,'.')"/>
    </xsl:if>
    <!-- если миллисекунд нет - возврат всей строки -->
    <xsl:if test="not($isMilliseconds)">
      <xsl:value-of select="$time"/>
    </xsl:if>
  </xsl:template>

  <xsl:template name="LocalDateFormat">
    <!-- форматирование даты к виду "ДД.ММ.ГГГ" -->
    <xsl:param name="date"/>
    <xsl:variable name='firstMinus' select="starts-with($date,'-')"/>
    <xsl:if test="$firstMinus">
      <!-- есть первый минус - до Р.Х. -->
      <xsl:text>до Р.Х. </xsl:text>
      <!-- формат даты -->
      <xsl:call-template name="LocalDateFormat2">
        <xsl:with-param name="date" select="substring-after($date,'-')"/>
      </xsl:call-template>
    </xsl:if>
    <xsl:if test="not($firstMinus)">
      <!-- формат даты -->
      <xsl:call-template name="LocalDateFormat2">
        <xsl:with-param name="date" select="$date"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>

  <xsl:template name="LocalDateFormat2">
    <!-- форматирование даты к виду "ДД.ММ.ГГГ" -->
    <xsl:param name="date"/>
    <xsl:variable name='year' select="substring-before($date,'-')"/>
    <xsl:variable name='monthDay' select="substring-after($date,'-')"/>
    <xsl:variable name='month' select="substring-before($monthDay,'-')"/>
    <xsl:variable name='day' select="substring-after($monthDay,'-')"/>
    <xsl:value-of select="concat($day,'.',$month,'.',$year)"/>
  </xsl:template>

</xsl:stylesheet>
